#!/usr/bin/env python3
"""
Trading Engine - Handles order execution, position tracking, and trading logic.
"""

import os
import json
import logging
from typing import Dict, Any, Optional, Tuple, List
from datetime import datetime

from src.config.config import Config
from src.clients.hyperliquid_client import HyperliquidClient
from src.trading.trading_stats import TradingStats

logger = logging.getLogger(__name__)

class TradingEngine:
    """Handles all trading operations, order execution, and position tracking."""
    
    def __init__(self):
        """Initialize the trading engine."""
        self.client = HyperliquidClient()
        self.stats = None
        
        # State persistence
        self.state_file = "trading_engine_state.json"
        
        # Position and order tracking
        self.bot_trade_ids = set()  # Track bot-generated trades
        self.pending_stop_losses = {}  # Pending stop loss orders
        
        # Load state and initialize stats
        self._load_state()
        self._initialize_stats()
        
    def _load_state(self):
        """Load trading engine state from disk."""
        try:
            if os.path.exists(self.state_file):
                with open(self.state_file, 'r') as f:
                    state_data = json.load(f)
                
                self.bot_trade_ids = set(state_data.get('bot_trade_ids', []))
                self.pending_stop_losses = state_data.get('pending_stop_losses', {})
                
                logger.info(f"🔄 Loaded trading engine state: {len(self.bot_trade_ids)} tracked trades")
        except Exception as e:
            logger.error(f"Error loading trading engine state: {e}")
            self.bot_trade_ids = set()
            self.pending_stop_losses = {}
    
    def _save_state(self):
        """Save trading engine state to disk."""
        try:
            state_data = {
                'bot_trade_ids': list(self.bot_trade_ids),
                'pending_stop_losses': self.pending_stop_losses,
                'last_updated': datetime.now().isoformat()
            }
            
            with open(self.state_file, 'w') as f:
                json.dump(state_data, f, indent=2, default=str)
        except Exception as e:
            logger.error(f"Error saving trading engine state: {e}")
    
    def _initialize_stats(self):
        """Initialize trading statistics."""
        try:
            self.stats = TradingStats()
            
            # Set initial balance
            balance = self.client.get_balance()
            if balance and balance.get('total'):
                usdc_balance = float(balance['total'].get('USDC', 0))
                self.stats.set_initial_balance(usdc_balance)
        except Exception as e:
            logger.error(f"Could not initialize trading stats: {e}")
    
    def get_balance(self) -> Optional[Dict[str, Any]]:
        """Get account balance."""
        return self.client.get_balance()
    
    def get_positions(self) -> Optional[List[Dict[str, Any]]]:
        """Get current positions."""
        return self.client.get_positions()
    
    def get_orders(self) -> Optional[List[Dict[str, Any]]]:
        """Get open orders."""
        return self.client.get_open_orders()
    
    def get_market_data(self, symbol: str) -> Optional[Dict[str, Any]]:
        """Get market data for a symbol."""
        return self.client.get_market_data(symbol)
    
    def find_position(self, token: str) -> Optional[Dict[str, Any]]:
        """Find an open position for a token."""
        symbol = f"{token}/USDC:USDC"
        positions = self.get_positions()
        
        if positions:
            for position in positions:
                if (position.get('symbol') == symbol and 
                    float(position.get('contracts', 0)) != 0):
                    return position
        return None
    
    def get_position_direction(self, position: Dict[str, Any]) -> Tuple[str, str, float]:
        """
        Get position direction info from CCXT position data.
        Returns: (position_type, exit_side, contracts_abs)
        """
        contracts = float(position.get('contracts', 0))
        side_field = position.get('side', '').lower()
        
        if side_field == 'long':
            return "LONG", "sell", abs(contracts)
        elif side_field == 'short':
            return "SHORT", "buy", abs(contracts)
        else:
            # Fallback to contracts sign (less reliable)
            if contracts > 0:
                return "LONG", "sell", abs(contracts)
            else:
                return "SHORT", "buy", abs(contracts)
    
    async def execute_long_order(self, token: str, usdc_amount: float, 
                                limit_price_arg: Optional[float] = None,
                                stop_loss_price: Optional[float] = None) -> Dict[str, Any]:
        symbol = f"{token}/USDC:USDC"
        
        try:
            # Validate inputs
            if usdc_amount <= 0:
                return {"success": False, "error": "Invalid USDC amount"}
            
            # Get market data for price validation
            market_data = self.get_market_data(symbol)
            if not market_data or not market_data.get('ticker'):
                return {"success": False, "error": f"Could not fetch market data for {token}"}
            
            current_price = float(market_data['ticker'].get('last', 0.0) or 0.0)
            if current_price <= 0:
                # Allow trading if current_price is 0 but a valid limit_price_arg is provided
                if not (limit_price_arg and limit_price_arg > 0):
                    return {"success": False, "error": f"Invalid current price ({current_price}) for {token} and no valid limit price provided."}
            
            order_placement_price: float
            token_amount: float

            if limit_price_arg is not None:
                # Limit order intent
                if limit_price_arg <= 0:
                    return {"success": False, "error": "Limit price must be positive."}
                order_placement_price = limit_price_arg
                token_amount = usdc_amount / order_placement_price
                logger.info(f"Placing LIMIT BUY order for {token_amount:.6f} {symbol} at ${order_placement_price:,.2f}")
                order = self.client.place_limit_order(symbol, 'buy', token_amount, order_placement_price)
            else:
                # Market order intent
                if current_price <= 0: # Re-check specifically for market order if current_price was initially 0
                    return {"success": False, "error": f"Cannot place market order for {token} due to invalid current price: {current_price}"}
                order_placement_price = current_price 
                token_amount = usdc_amount / order_placement_price
                logger.info(f"Placing MARKET BUY order for {token_amount:.6f} {symbol} (approx. price ${order_placement_price:,.2f})")
                order = self.client.place_market_order(symbol, 'buy', token_amount)
            
            if not order:
                logger.error(f"Order placement call failed for {symbol}. Client returned None or empty.")
                return {"success": False, "error": "Order placement failed at client level (no order object)."}
            
            order_id = order.get('id', 'N/A')
            order_avg_fill_price = order.get('average') 

            # For stats, use average fill price if available, otherwise the price order was placed at.
            final_price_for_stats = order_avg_fill_price if order_avg_fill_price is not None else order_placement_price

            if final_price_for_stats is None: 
                logger.critical(f"CRITICAL: final_price_for_stats is None for order {order_id}. Order: {order}, Placement Price: {order_placement_price}, Avg Fill: {order_avg_fill_price}")
                return {"success": False, "error": "Critical: Price for stats recording became None."}

            logger.info(f"Order {order_id} for {symbol} processing. AvgFill: {order_avg_fill_price}, PlacementPrice: {order_placement_price}, StatsPrice: {final_price_for_stats}")
            
            if order_id != 'N/A':
                self.bot_trade_ids.add(order_id)
            
            # Record in stats
            action_type = self.stats.record_trade_with_enhanced_tracking(
                symbol, 'buy', token_amount, final_price_for_stats, order_id, "bot"
            )
            
            # Handle stop loss if specified
            if stop_loss_price and order_id != 'N/A':
                self.pending_stop_losses[order_id] = {
                    'token': token,
                    'stop_price': stop_loss_price,
                    'side': 'sell', 
                    'amount': token_amount,
                    'order_type': 'stop_loss'
                }
            
            self._save_state()
            
            return {
                "success": True,
                "order": order,
                "action_type": action_type,
                "token_amount": token_amount,
                "actual_price": final_price_for_stats,
                "stop_loss_pending": stop_loss_price is not None
            }
        except ZeroDivisionError as e:
            logger.error(f"Error executing long order due to ZeroDivisionError (likely price issue): {e}. LimitArg: {limit_price_arg}, CurrentPrice: {current_price if 'current_price' in locals() else 'N/A'}")
            return {"success": False, "error": f"Math error (division by zero), check prices: {e}"}
        except Exception as e:
            logger.error(f"Error executing long order: {e}", exc_info=True)
            return {"success": False, "error": str(e)}
    
    async def execute_short_order(self, token: str, usdc_amount: float, 
                                 limit_price_arg: Optional[float] = None,
                                 stop_loss_price: Optional[float] = None) -> Dict[str, Any]:
        symbol = f"{token}/USDC:USDC"
        
        try:
            if usdc_amount <= 0:
                return {"success": False, "error": "Invalid USDC amount"}
            
            market_data = self.get_market_data(symbol)
            if not market_data or not market_data.get('ticker'):
                return {"success": False, "error": f"Could not fetch market data for {token}"}
            
            current_price = float(market_data['ticker'].get('last', 0.0) or 0.0)
            if current_price <= 0:
                if not (limit_price_arg and limit_price_arg > 0):
                    return {"success": False, "error": f"Invalid current price ({current_price}) for {token} and no valid limit price provided."}

            order_placement_price: float
            token_amount: float

            if limit_price_arg is not None:
                if limit_price_arg <= 0:
                    return {"success": False, "error": "Limit price must be positive."}
                order_placement_price = limit_price_arg
                token_amount = usdc_amount / order_placement_price
                logger.info(f"Placing LIMIT SELL order for {token_amount:.6f} {symbol} at ${order_placement_price:,.2f}")
                order = self.client.place_limit_order(symbol, 'sell', token_amount, order_placement_price)
            else:
                if current_price <= 0:
                     return {"success": False, "error": f"Cannot place market order for {token} due to invalid current price: {current_price}"}
                order_placement_price = current_price
                token_amount = usdc_amount / order_placement_price
                logger.info(f"Placing MARKET SELL order for {token_amount:.6f} {symbol} (approx. price ${order_placement_price:,.2f})")
                order = self.client.place_market_order(symbol, 'sell', token_amount)
            
            if not order:
                logger.error(f"Order placement call failed for {symbol}. Client returned None or empty.")
                return {"success": False, "error": "Order placement failed at client level (no order object)."}

            order_id = order.get('id', 'N/A')
            order_avg_fill_price = order.get('average')
            
            final_price_for_stats = order_avg_fill_price if order_avg_fill_price is not None else order_placement_price

            if final_price_for_stats is None:
                logger.critical(f"CRITICAL: final_price_for_stats is None for order {order_id}. Order: {order}, Placement Price: {order_placement_price}, Avg Fill: {order_avg_fill_price}")
                return {"success": False, "error": "Critical: Price for stats recording became None."}

            logger.info(f"Order {order_id} for {symbol} processing. AvgFill: {order_avg_fill_price}, PlacementPrice: {order_placement_price}, StatsPrice: {final_price_for_stats}")

            if order_id != 'N/A':
                self.bot_trade_ids.add(order_id)
            
            action_type = self.stats.record_trade_with_enhanced_tracking(
                symbol, 'sell', token_amount, final_price_for_stats, order_id, "bot"
            )
            
            if stop_loss_price and order_id != 'N/A':
                self.pending_stop_losses[order_id] = {
                    'token': token,
                    'stop_price': stop_loss_price,
                    'side': 'buy', # Exit side for short
                    'amount': token_amount,
                    'order_type': 'stop_loss'
                }
            
            self._save_state()
            
            return {
                "success": True,
                "order": order,
                "action_type": action_type,
                "token_amount": token_amount,
                "actual_price": final_price_for_stats,
                "stop_loss_pending": stop_loss_price is not None
            }
        except ZeroDivisionError as e:
            logger.error(f"Error executing short order due to ZeroDivisionError (likely price issue): {e}. LimitArg: {limit_price_arg}, CurrentPrice: {current_price if 'current_price' in locals() else 'N/A'}")
            return {"success": False, "error": f"Math error (division by zero), check prices: {e}"}
        except Exception as e:
            logger.error(f"Error executing short order: {e}", exc_info=True)
            return {"success": False, "error": str(e)}
    
    async def execute_exit_order(self, token: str) -> Dict[str, Any]:
        """Execute an exit order to close a position."""
        position = self.find_position(token)
        if not position:
            return {"success": False, "error": f"No open position found for {token}"}
        
        try:
            symbol = f"{token}/USDC:USDC"
            position_type, exit_side, contracts = self.get_position_direction(position)
            
            # Execute market order to close position
            order = self.client.place_market_order(symbol, exit_side, contracts)
            
            if order:
                order_id = order.get('id', 'N/A')
                actual_price = order.get('average', 0)
                
                if order_id != 'N/A':
                    self.bot_trade_ids.add(order_id)
                
                action_type = self.stats.record_trade_with_enhanced_tracking(
                    symbol, exit_side, contracts, actual_price, order_id, "bot"
                )
                
                self._save_state()
                
                return {
                    "success": True,
                    "order": order,
                    "action_type": action_type,
                    "position_type": position_type,
                    "contracts": contracts,
                    "actual_price": actual_price
                }
            else:
                return {"success": False, "error": "Exit order execution failed"}
                
        except Exception as e:
            logger.error(f"Error executing exit order: {e}")
            return {"success": False, "error": str(e)}
    
    async def execute_stop_loss_order(self, token: str, stop_price: float) -> Dict[str, Any]:
        """Execute a stop loss order."""
        position = self.find_position(token)
        if not position:
            return {"success": False, "error": f"No open position found for {token}"}
        
        try:
            symbol = f"{token}/USDC:USDC"
            position_type, exit_side, contracts = self.get_position_direction(position)
            entry_price = float(position.get('entryPrice', 0))
            
            # Validate stop loss price based on position direction
            if position_type == "LONG" and stop_price >= entry_price:
                return {"success": False, "error": "Stop loss price should be below entry price for long positions"}
            elif position_type == "SHORT" and stop_price <= entry_price:
                return {"success": False, "error": "Stop loss price should be above entry price for short positions"}
            
            # Place limit order at stop loss price
            order = self.client.place_limit_order(symbol, exit_side, contracts, stop_price)
            
            if order:
                order_id = order.get('id', 'N/A')
                
                if order_id != 'N/A':
                    self.bot_trade_ids.add(order_id)
                
                self._save_state()
                
                return {
                    "success": True,
                    "order": order,
                    "position_type": position_type,
                    "contracts": contracts,
                    "stop_price": stop_price
                }
            else:
                return {"success": False, "error": "Stop loss order placement failed"}
                
        except Exception as e:
            logger.error(f"Error executing stop loss order: {e}")
            return {"success": False, "error": str(e)}
    
    async def execute_take_profit_order(self, token: str, profit_price: float) -> Dict[str, Any]:
        """Execute a take profit order."""
        position = self.find_position(token)
        if not position:
            return {"success": False, "error": f"No open position found for {token}"}
        
        try:
            symbol = f"{token}/USDC:USDC"
            position_type, exit_side, contracts = self.get_position_direction(position)
            entry_price = float(position.get('entryPrice', 0))
            
            # Validate take profit price based on position direction
            if position_type == "LONG" and profit_price <= entry_price:
                return {"success": False, "error": "Take profit price should be above entry price for long positions"}
            elif position_type == "SHORT" and profit_price >= entry_price:
                return {"success": False, "error": "Take profit price should be below entry price for short positions"}
            
            # Place limit order at take profit price
            order = self.client.place_limit_order(symbol, exit_side, contracts, profit_price)
            
            if order:
                order_id = order.get('id', 'N/A')
                
                if order_id != 'N/A':
                    self.bot_trade_ids.add(order_id)
                
                self._save_state()
                
                return {
                    "success": True,
                    "order": order,
                    "position_type": position_type,
                    "contracts": contracts,
                    "profit_price": profit_price
                }
            else:
                return {"success": False, "error": "Take profit order placement failed"}
                
        except Exception as e:
            logger.error(f"Error executing take profit order: {e}")
            return {"success": False, "error": str(e)}
    
    async def cancel_all_orders(self, token: str) -> Dict[str, Any]:
        """Cancel all orders for a token."""
        try:
            symbol = f"{token}/USDC:USDC"
            result = self.client.cancel_all_orders(symbol)
            
            return {
                "success": True,
                "result": result
            }
        except Exception as e:
            logger.error(f"Error cancelling orders: {e}")
            return {"success": False, "error": str(e)}
    
    # Alias methods for consistency with command handlers
    async def execute_sl_order(self, token: str, stop_price: float) -> Dict[str, Any]:
        """Alias for execute_stop_loss_order."""
        return await self.execute_stop_loss_order(token, stop_price)
    
    async def execute_tp_order(self, token: str, profit_price: float) -> Dict[str, Any]:
        """Alias for execute_take_profit_order."""
        return await self.execute_take_profit_order(token, profit_price)
    
    async def execute_coo_order(self, token: str) -> Dict[str, Any]:
        """Alias for cancel_all_orders."""
        return await self.cancel_all_orders(token)
    
    def is_bot_trade(self, trade_id: str) -> bool:
        """Check if a trade was generated by this bot."""
        return trade_id in self.bot_trade_ids
    
    def get_stats(self) -> TradingStats:
        """Get trading statistics object."""
        return self.stats 